from Module import Module
from irclib import irclib
from ConfigParser import NoOptionError
from datetime import datetime
from dateutil.tz import *

class Seen(Module):
    # Keeps track of when people say things in public channels, and reports on when
    # they last said something.

    def __init__(self, config, server, modlist, rehash):
        super(Seen, self).__init__(config, server, modlist, rehash)
        modlist.append(self)

    def register_handlers(self, server):
        server.add_global_handler('pubmsg', self.on_pubmsg)
        server.add_global_handler('privmsg', self.on_privmsg)

    def do(self, connection, event, nick, userhost, replypath, what, admin_unlocked):
        # whatever it is, store it
        if not self.config.has_section('seen'):
            self.config.add_section('seen')

        self.config.set('seen', nick, userhost + '|:|' + datetime.utcnow().isoformat() + '|:|' + what)

        # also see if it's a query
        whats = what.split(' ')
        if whats[0] == 'seen' and len(whats) >= 2:
            query = whats[1]
            if query != 'debug':
                try:
                    seendata = self.config.get('seen', query).split('|:|')
                    converted = datetime.strptime(seendata[1], "%Y-%m-%dT%H:%M:%S.%f").replace(tzinfo=tzutc())
                    replystr = 'last saw ' + query + ' at ' + converted.astimezone(tzlocal()).strftime("%Y/%m/%d %H:%M:%S %Z") + ' saying \'' + seendata[2] + '\''
                    if replypath is None:
                        return replystr
                    else:
                        connection.privmsg(replypath, replystr)
                except NoOptionError: pass

